const { spawn } = require('child_process')
const { Lock } = require('semaphore-async-await')


function PythonWorker(workerScript, shellOptions) {
  // create a lock for incomming commands
  this.commandLock = new Lock()
  // create a lock for python communication
  this.pythonLock = new Lock()

  this.data = []
  this.error = []
  this.workerScript = workerScript

  // The python shell is started with this.spawn
  this.pythonShell = null

  // Use send a command to the python worker.
  // 
  this.send = async (command) => {
    if (this.pythonShell.killed) return { error: `process ${this.pythonShell.pid} already killed.` }
    if (this.pythonShell.exitCode !== null) return { error: `process ${this.pythonShell.pid} already exited with code ${this.pythonShell.exitCode}` }
    await this.commandLock.acquire()
    await this.pythonLock.acquire()
    console.log(command)
    this.pythonShell.stdin.write(
      // Write the command as a JSON object. 
      //!Don't forget the new-line
      JSON.stringify(command) + '\n'
    )
    await this.pythonLock.acquire()
    const pythonError = this.error.pop()
    if (pythonError) {
      this.pythonLock.release()
      this.commandLock.release()
      console.log({ pythonError })
      return { pythonError }
    }
    const { data, error } = this.data.pop()
    this.pythonLock.release()
    this.commandLock.release()
    console.log({ data, error })
    return { data, error }
  }

  this.spawn = async () => {
    if (this.pythonShell) {
      if (!this.pythonShell.killed && (this.pythonShell.exitCode === null)) {
        return { error: `process still running with pid = ${this.pythonShell.pid}` }
      }
    }
    await this.commandLock.acquire()
    await this.pythonLock.acquire()
    this.pythonShell = spawn(
      `${process.env.PWD}/${process.env.PYTHON_PATH}`,
      [this.workerScript]
    )
    this.pythonShell.stdout.on('data', message => {
      this.data.push(JSON.parse(message))
      this.pythonLock.release()
    })
    this.pythonShell.stderr.on('data', error => {
      const jsonError = JSON.parse(error)
      this.error.push({ error: jsonError })
      this.pythonLock.release()
    })
    this.pythonShell.on('close', error => {
      const jsonError = JSON.parse(error)
      this.error.push({ error: jsonError })
      this.pythonLock.release()
    })
    await this.pythonLock.acquire()
    const pythonError = this.error.pop()
    if (pythonError) {
      this.pythonLock.release()
      this.commandLock.release()
      return { pythonError }
    }
    const { data, error } = this.data.pop()
    this.pythonLock.release()
    this.commandLock.release()
    return { data, error }
  }

  this.end = async () => {
    if (this.pythonShell.killed) return { error: `process ${this.pythonShell.pid} already killed.` }
    if (this.pythonShell.exitCode !== null) return { error: `process ${this.pythonShell.pid} already exited with code ${this.pythonShell.exitCode}` }
    await this.commandLock.acquire()
    await this.pythonLock.acquire()
    this.pythonShell.stdin.end()
    await this.pythonLock.acquire()
    const pythonError = this.error.pop()
    if (pythonError) {
      this.pythonLock.release()
      this.commandLock.release()
      return { pythonError }
    }
    const { data, error } = this.data.pop()
    this.pythonLock.release()
    this.commandLock.release()
    return { data, error }
  }

  this.kill = async () => {
    if (this.pythonShell.killed) return { error: `process ${this.pythonShell.pid} already killed.` }
    if (this.pythonShell.exitCode !== null) return { error: `process ${this.pythonShell.pid} already exited with code ${this.pythonShell.exitCode}` }
    await this.commandLock.acquire()
    await this.pythonLock.acquire()
    const pythonError = this.error.pop()
    if (pythonError) {
      this.pythonLock.release()
      this.commandLock.release()
      return { pythonError }
    }
    const { data, error } = this.data.pop()
    this.pythonLock.release()
    this.commandLock.release()
    return { data, error }
  }
}

module.exports = PythonWorker
